// Copyright (c) 2011,2016 Douglas Miller

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.sound.sampled.*;
import java.util.Date;
import java.util.Calendar;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.awt.datatransfer.StringSelection;

class gc1005_Cabinet extends JPanel {
	static final long serialVersionUID = 311100500008L;

	private static final Point[] layout = new Point[]{
		new Point(2,0),
		new Point(8,0),
		new Point(104,0),
		new Point(110,0),
		new Point(4,2),
		new Point(108,2),
		new Point(4,5),
		new Point(108,5),
		new Point(4,28),
		new Point(108,28),
		new Point(4,36),
		new Point(108,36),
		new Point(0,36),
		new Point(6,36),
		new Point(106,36),
		new Point(112,36),
		new Point(0,44),
		new Point(6,44),
		new Point(106,44),
		new Point(112,44),
	};
	private static final int width = 112;
	private static final int height = 44;

	private Point[] scaled;

	Polygon side_L;
	Polygon side_R;
	Polygon top;
	Polygon wind;
	Polygon nameplate;
	Polygon foot_L;
	Polygon foot_R;
	Polygon base;
	Rectangle plate;
	static final String platetext = "HEATHKIT";
	int namex;
	int namey;
	Font font;

	Dimension size;

	private void drawLine(Graphics2D g2d, int p1, int p2) {
		g2d.drawLine(scaled[p1].x, scaled[p1].y,
			scaled[p2].x, scaled[p2].y);
	}

	public void paint(Graphics g) {
		Graphics2D g2d = (Graphics2D)g;
		// can't seen to get anitalias to work right.
		// it is as if TEXT is swapped with regular...
		//g2d.setRenderingHint(
		//	RenderingHints.KEY_ANTIALIASING,
		//	RenderingHints.VALUE_ANTIALIAS_ON);
		// setting regular ruins antialiasing?
		g2d.setRenderingHint(
			RenderingHints.KEY_TEXT_ANTIALIASING,
			RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		//g2d.addRenderingHints(new RenderingHints(
		//	RenderingHints.KEY_ANTIALIASING,
		//	RenderingHints.VALUE_ANTIALIAS_ON));
		//g2d.setRenderingHints(new RenderingHints(
		//	RenderingHints.KEY_ANTIALIASING,
		//	RenderingHints.VALUE_ANTIALIAS_ON));

		super.paint(g2d);

		g2d.setColor(Heath_Colors.black2);
		g2d.fillPolygon(top);
		g2d.setColor(Heath_Colors.empty);
		g2d.fillPolygon(wind);
		g2d.setColor(Heath_Colors.cabinet);
		g2d.fillPolygon(nameplate);
		g2d.setColor(Heath_Colors.highlight);
		drawLine(g2d, 4, 5);
		drawLine(g2d, 8, 9);

		g2d.setColor(Heath_Colors.black1);
		g2d.fillPolygon(side_L);
		g2d.fillPolygon(side_R);
		g2d.setColor(Color.black);
		g2d.fillRect(plate.x, plate.y, plate.width, plate.height);
		g2d.setColor(Heath_Colors.black2);
		g2d.fillPolygon(foot_L);
		g2d.fillPolygon(foot_R);
		g2d.setColor(Heath_Colors.black3);
		g2d.fillPolygon(base);

		g2d.setColor(Heath_Colors.highlight);
		drawLine(g2d, 0, 12);
		drawLine(g2d, 0, 1);
		drawLine(g2d, 2, 3);
		drawLine(g2d, 2, 14);
		drawLine(g2d, 12, 13);
		drawLine(g2d, 14, 15);
		drawLine(g2d, 12, 16);
		drawLine(g2d, 14, 18);

		g2d.setColor(Color.white);
		g2d.setFont(font);
		g2d.drawString(platetext, namex, namey);
	}

	private Polygon getPolygon(int... pts) {
		int[] x = new int[pts.length];
		int[] y = new int[pts.length];
		int j = 0;
		for (int i : pts) {
			x[j] = scaled[i].x;
			y[j] = scaled[i].y;
			++j;
		}
		return new Polygon(x, y, x.length);
	}

	public void scale(float factor) {
		scaled = new Point[layout.length];
		for (int x = 0; x < layout.length; ++x) {
			scaled[x] = new Point(Math.round(layout[x].x * factor),
					Math.round(layout[x].y * factor));
		}
		size = new Dimension(Math.round(width * factor),
				Math.round(height * factor));
		side_L = getPolygon(0, 1, 13, 12);
		side_R = getPolygon(2, 3, 15, 14);
		top = getPolygon(4, 5, 7, 6);
		wind = getPolygon(6, 7, 9, 8);
		nameplate = getPolygon(8, 9, 11, 10);
		foot_L = getPolygon(12, 13, 17, 16);
		foot_R = getPolygon(14, 15, 19, 18);
		base = getPolygon(13, 14, 18, 17);
		Rectangle r = nameplate.getBounds();
		int h = Math.round(4 * factor);
		
		plate = new Rectangle(r.x + Math.round(4 * factor),
			r.y + (r.height - h) / 2,
			Math.round(24 * factor), h);
		// todo: scale font...
		font = new Font("Serif", Font.PLAIN, 10);
		FontMetrics fm = getFontMetrics(font);
		int w = fm.stringWidth(platetext);
		int b = fm.getAscent();
		namex = plate.x + (plate.width - w) / 2;
		namey = plate.y + (plate.height - b) / 2 + b - (int)Math.round(b * 0.1);
	}

	public gc1005_Cabinet() {
		scale(3.125f);
		setPreferredSize(size);
		setBackground(new Color(0, 0, 0, 0));
		setOpaque(false);
	}

	public Rectangle getWindow() {
		return wind.getBounds();
	}

	public Dimension getSize() {
		return size;
	}
}
