// Copyright (c) 2011,2016 Douglas Miller

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.sound.sampled.*;
import java.util.Date;
import java.util.Calendar;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.awt.datatransfer.StringSelection;

class gc1005_Clock extends JLayeredPane
		implements MouseListener, MouseMotionListener, ActionListener {
	static final long serialVersionUID = 311100500009L;

	gc1005_Cabinet _cab;
	gc1005_Display _disp;
	gc1005_Properties _props;
	JPopupMenu popup = new JPopupMenu("Edit");

	JFrame _top;
	int _press_x;
	int _press_y;
	boolean screensaver = false;
	boolean xscreensaver = false;
	javax.swing.Timer timer;
	int dx = 0;
	int dy = 0;

	public gc1005_Clock(gc1005_Properties props, JFrame top,
			gc1005_Cabinet cab, gc1005_Display disp, gc1005_NeBulb bulb) {
		_props = props;
		_top = top;
		_cab = cab;
		_disp = disp;
		int layer = 1;
		Dimension z = cab.getSize();
		Rectangle w = cab.getWindow();
		setPreferredSize(z);
		setOpaque(false);
		Dimension d = disp.getPreferredSize();
		Point r = new Point(w.x + (w.width - d.width) / 2,
				w.y + (w.height - d.height) / 2);
		Point b = new Point(r.x - 10,
				w.y + (w.height - 30) / 2);
		disp.setBounds(r.x, r.y, d.width, d.height);
		cab.setBounds(0, 0, z.width, z.height);
		bulb.setBounds(b.x, b.y, 11, 31);
		add(cab, new Integer(layer), layer);
		++layer;
		add(disp, new Integer(layer), layer);
		++layer;
		add(bulb, new Integer(layer), layer);
		++layer;

		screensaver = _props.getBoolean("gc1005_screensaver");
		xscreensaver = _props.getBoolean("gc1005_xscreensaver");
		if (screensaver) {
			int time = 5000;
			dx = 350;
			dy = 135;
			if (true) {
				time = 1000;
				dx = 20;
				dy = 5;
			}
			if (!xscreensaver) {
				addMouseMotionListener(this);
			}
			timer = new Timer(time, this);
			timer.start();
		} else {
			for (JMenuItem mi : disp.getMenu()) {
				mi.addActionListener(this);
				popup.add(mi);
			}
			JMenuItem mi;
			mi = new JMenuItem("Copy", KeyEvent.VK_COPY);
			KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK);
			mi.setAccelerator(ks);
			mi.addActionListener(this);
			popup.add(mi);
			mi = new JMenuItem("Preferences", KeyEvent.VK_E);
			mi.addActionListener(this);
			popup.add(mi);
			mi = new JMenuItem("Quit", KeyEvent.VK_Q);
			mi.addActionListener(this);
			popup.add(mi);
		}
		if (!xscreensaver) {
			addMouseListener(this);
		}
	}

	public void actionPerformed(ActionEvent e) {
	        if (e.getSource() instanceof Timer) {
			Dimension s = getParent().getSize();
			Rectangle b = getBounds();
			if (b.x + dx + b.width >= s.width || b.x + dx < 0) dx = -dx;
			b.x += dx;
			if (b.y + dy + b.height >= s.height || b.y + dy < 0) dy = -dy;
			b.y += dy;
			setBounds(b);
			return;
	        } else if (!(e.getSource() instanceof JMenuItem)) {
	                return;
	        }
		JMenuItem m = (JMenuItem)e.getSource();
		if (m.getMnemonic() == KeyEvent.VK_Z) {
			_disp.alarmSnooze();
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_R) {
			_disp.alarmStop();
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_S) {
			_disp.alarmSet();
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_T) {
			_disp.timeSet();
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_A) {
			boolean on = !_disp.alarm();
			try { // if this fails, oh well.
				_props.setAndSaveProperty(
					new gc1005_Properties(),
					"gc1005_alarm_on", Boolean.toString(on));
			} catch(Exception ee) { ee.printStackTrace(); }
			_disp.alarm(on);
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_COPY) {
			_disp.copy();
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_E) {
			boolean changed = _props.editPreferences();
			if (changed) {
				// Apply properties...
				_disp.setProperties(_props);
				// <others>.setProperties(_props);
			}
			return;
		}
		if (m.getMnemonic() == KeyEvent.VK_Q) {
			System.exit(0);
			return;
		}
	}

	public void mouseClicked(MouseEvent e) {
		if (screensaver) {
			System.exit(0);
		}
		if (e.getButton() == MouseEvent.BUTTON3) {
			popup.show(this, e.getX(), e.getY());
		}
	}
	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mousePressed(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }

	public void mouseMoved(MouseEvent e) {
		if (screensaver) {
			System.exit(0);
		}
	}
	public void mouseDragged(MouseEvent e) { }
}
