// Copyright (c) 2011,2014 Douglas Miller
// $Id: gc1005_Properties.java,v 1.6 2014/01/27 21:12:45 drmiller Exp $

import java.awt.*;
import javax.swing.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.FileNotFoundException;

class gc1005_Properties extends Heath_Properties
		implements Heath_PropertyEditor
{
	static final long serialVersionUID = 311000000015L;
	JCheckBox _24h_cb;
	JCheckBox _auto_cb;
	JCheckBox _sp1_cb;
	JRadioButton _f_rb1;
	JRadioButton _f_rb2;
	JRadioButton _f_rb3;
	ButtonGroup _f_bg;
	JLabel _f_lb;
	JTextArea _alarm_tx;
	JPanel _alarm_pn;
	JCheckBox _alm_cb;
	JTextArea _alarm_vol_tx;
	JPanel _alarm_vol_pn;
	JPanel _dia_pn;

	public gc1005_Properties() {
		try {
			initProperties("gc1005", "~/.gc1005.rc");
		} catch (Exception e) {
			if (e instanceof FileNotFoundException) {
				// Don't complain about non-existent file,
				// just work with in-memory properties.
				// Still might not be possible to create...
			} else {
				warning("Load Setup", e.getMessage());
			}
		}
		processDefaults();

		// Edit Properties...
		_24h_cb = new JCheckBox("24-Hour Format");
		_auto_cb = new JCheckBox("Auto Time-Sync");
		_sp1_cb = new JCheckBox("Enable PanaPlex '1'");
		_alm_cb = new JCheckBox("Alarm On");
		_f_rb1 = new JRadioButton("PanaPlex 9-Segment");
		_f_rb1.setActionCommand("Panaplex9seg.ttf");
		_f_rb2 = new JRadioButton("Nixie Tubes");
		_f_rb2.setActionCommand("NixieZM1336.ttf");
		_f_rb3 = new JRadioButton("(not set)");
		_f_rb3.setActionCommand("nothing");
		_f_rb3.setEnabled(false);
		_f_bg = new ButtonGroup();
		_f_bg.add(_f_rb1);
		_f_bg.add(_f_rb2);
		_f_bg.add(_f_rb3);
		_f_lb = new JLabel("Display style:");
		_alarm_tx = new JTextArea();
		_alarm_tx.setPreferredSize(new Dimension(200, 20));
		_alarm_pn = new JPanel();
		_alarm_pn.add(new JLabel("Alarm:"));
		_alarm_pn.add(_alarm_tx);

		_alarm_vol_tx = new JTextArea();
		_alarm_vol_tx.setPreferredSize(new Dimension(100, 20));
		_alarm_vol_pn = new JPanel();
		_alarm_vol_pn.add(new JLabel("Alarm Volume (%)"));
		_alarm_vol_pn.add(_alarm_vol_tx);

		_dia_pn = new JPanel();
		GridBagLayout gridbag = new GridBagLayout();
		_dia_pn.setLayout(gridbag);
		GridBagConstraints s = new GridBagConstraints();
		s.fill = GridBagConstraints.NONE;
		s.gridx = 1;
		s.gridy = 1;
		s.weightx = 1;
		s.weighty = 1;
		s.gridwidth = 1;
		s.gridheight = 1;
		s.insets.left = 0;
		s.insets.right = 0;
		s.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(_24h_cb, s);
		_dia_pn.add(_24h_cb);
		s.gridy += 1;
		gridbag.setConstraints(_alarm_pn, s);
		_dia_pn.add(_alarm_pn);
		s.gridy += 1;
		gridbag.setConstraints(_alm_cb, s);
		_dia_pn.add(_alm_cb);
		s.gridy += 1;
		gridbag.setConstraints(_alarm_vol_pn, s);
		_dia_pn.add(_alarm_vol_pn);
		s.gridy += 1;
		gridbag.setConstraints(_auto_cb, s);
		_dia_pn.add(_auto_cb);
		s.gridy += 1;
		gridbag.setConstraints(_sp1_cb, s);
		_dia_pn.add(_sp1_cb);
		s.gridy += 1;
		gridbag.setConstraints(_f_lb, s);
		_dia_pn.add(_f_lb);
		s.gridy += 1;
		gridbag.setConstraints(_f_rb1, s);
		_dia_pn.add(_f_rb1);
		s.gridy += 1;
		gridbag.setConstraints(_f_rb2, s);
		_dia_pn.add(_f_rb2);
		s.gridy += 1;
		gridbag.setConstraints(_f_rb3, s);
		_dia_pn.add(_f_rb3);
		setupDialog(_dia_pn, null); // TODO: icon?
	}

	public void processDefaults() {
		int changes = 0;
		// setup defaults for everything...
		String s;
		s = getProperty("gc1005_24hour");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_24hour", "false");
		}
		s = getProperty("gc1005_alarm");
		if (s == null || s.length() == 0) {
			++changes;
			if (getBoolean("gc1005_24hour")) {
				s = "00:00";
			} else {
				s = "12:00 AM";
			}
			setProperty("gc1005_alarm", s);
		}
		s = getProperty("gc1005_alarm_volume");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_alarm_volume", "50");
		}
		s = getProperty("gc1005_alarm_on");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_alarm_on", "false");
		}
		s = getProperty("gc1005_snooze");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_snooze", "7");
		}
		s = getProperty("gc1005_special1");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_special1", "false");
		}
		s = getProperty("gc1005_displayfont");
		if (s == null || s.length() == 0) {
			++changes;
			setProperty("gc1005_displayfont", "Panaplex9seg.ttf");
		}
		setProperty("gc1005_screensaver", "false");
		setProperty("gc1005_xscreensaver", "false");
		if (changes > 0) {
			_changed = true;
		}
	}

	public boolean editPreferences() {
		_sp1_cb.setSelected(getBoolean("gc1005_special1"));
		_24h_cb.setSelected(getBoolean("gc1005_24hour"));
		_auto_cb.setSelected(getBoolean("gc1005_auto"));
		_alm_cb.setSelected(getBoolean("gc1005_alarm_on"));
		String f = getProperty("gc1005_displayfont");
		if (f.equals(_f_rb1.getActionCommand())) {
			_f_rb1.setSelected(true);
		} else if (f.equals(_f_rb2.getActionCommand())) {
			_f_rb2.setSelected(true);
		} else {
			_f_rb3.setText(f);
			_f_rb3.setActionCommand(f);
			_f_rb3.setEnabled(true);
			_f_rb3.setSelected(true);
			// Need something user-editable...
		}
		_alarm_tx.setText(getProperty("gc1005_alarm"));
		_alarm_vol_tx.setText(getProperty("gc1005_alarm_volume"));

		int ret = doDialog();
		if (ret != OPTION_APPLY && ret != OPTION_SAVE) return false;

		// TBD: change parameters and restart?
		// TBD: do validation?
		ButtonModel bm = _f_bg.getSelection();
		setProperty("gc1005_displayfont", bm.getActionCommand());
		setProperty("gc1005_special1", Boolean.toString(_sp1_cb.isSelected()));
		setProperty("gc1005_24hour", Boolean.toString(_24h_cb.isSelected()));
		setProperty("gc1005_auto", Boolean.toString(_auto_cb.isSelected()));
		// regardless of what changed, force gc1005_24hour and gc1005_alarm
		// to match.
		boolean is24 = _24h_cb.isSelected();
		String alarm = _alarm_tx.getText();
		boolean amPm = alarm.matches(".*[AaPp][Mm].*");
		SimpleDateFormat f_24 = new SimpleDateFormat("HH:mm");
		SimpleDateFormat f_12 = new SimpleDateFormat("h:mm a");
		if (is24 && amPm) {
			try {
				Date dt = f_12.parse(alarm);
				alarm = f_24.format(dt);
			} catch (Exception e) {
				alarm = "";
			}
		} else if (!is24 && !amPm) {
			try {
				Date dt = f_24.parse(alarm);
				alarm = f_12.format(dt);
			} catch (Exception e) {
				alarm = "";
			}
		}
		setProperty("gc1005_alarm", alarm);
		setProperty("gc1005_alarm_on", Boolean.toString(_alm_cb.isSelected()));
		int vol = 50;
		try {
			vol = Integer.valueOf(_alarm_vol_tx.getText());
			if (vol < 0) { vol = 0; }
			if (vol > 100) { vol = 100; }
		} catch(Exception e) {
		}
		setProperty("gc1005_alarm_volume", Integer.toString(vol));
		processDefaults();

		if (ret == OPTION_SAVE) {
			try {
				save();
			} catch (Exception e) {
				warning("Save Setup", e.getMessage());
			}
		}
		return true;
	}

	static public void fatal(String op, String err) {
		JOptionPane.showMessageDialog(null,
			new JLabel(err),
			op + " Error", JOptionPane.ERROR_MESSAGE);
		System.exit(1);
	}

	static public void warning(String op, String err) {
		JOptionPane.showMessageDialog(null,
			new JLabel(err),
			op + " Warning", JOptionPane.WARNING_MESSAGE);
	}

	static public void inform(String op, String err) {
		JOptionPane.showMessageDialog(null,
			new JLabel(err),
			op + " Information", JOptionPane.INFORMATION_MESSAGE);
	}

	static public int confirm(String op, String err) {
		int res = JOptionPane.showConfirmDialog(null,
			new JLabel(err),
			op + " Confirmation", JOptionPane.YES_NO_OPTION);
		return res;
	}
}
